import { Injectable, EventEmitter, Component, Input, Output, ViewChild, Directive, HostBinding, HostListener, ElementRef, Pipe, NgModule } from '@angular/core';
import { isNil, pull, concat, isString, isBoolean, includes } from 'lodash';
import { Subject } from 'rxjs';
import { debounceTime, distinctUntilChanged } from 'rxjs/operators';
import { FormsModule } from '@angular/forms';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
class TreeviewI18n {
}
TreeviewI18n.decorators = [
    { type: Injectable }
];
if (false) {
    /**
     * @abstract
     * @param {?} selection
     * @return {?}
     */
    TreeviewI18n.prototype.getText = function (selection) { };
    /**
     * @abstract
     * @return {?}
     */
    TreeviewI18n.prototype.getAllCheckboxText = function () { };
    /**
     * @abstract
     * @return {?}
     */
    TreeviewI18n.prototype.getFilterPlaceholder = function () { };
    /**
     * @abstract
     * @return {?}
     */
    TreeviewI18n.prototype.getFilterNoItemsFoundText = function () { };
    /**
     * @abstract
     * @param {?} isCollapse
     * @return {?}
     */
    TreeviewI18n.prototype.getTooltipCollapseExpandText = function (isCollapse) { };
}
class TreeviewI18nDefault extends TreeviewI18n {
    /**
     * @param {?} selection
     * @return {?}
     */
    getText(selection) {
        if (selection.uncheckedItems.length === 0) {
            return this.getAllCheckboxText();
        }
        switch (selection.checkedItems.length) {
            case 0:
                return 'Select options';
            case 1:
                return selection.checkedItems[0].text;
            default:
                return `${selection.checkedItems.length} options selected`;
        }
    }
    /**
     * @return {?}
     */
    getAllCheckboxText() {
        return 'All';
    }
    /**
     * @return {?}
     */
    getFilterPlaceholder() {
        return 'Filter';
    }
    /**
     * @return {?}
     */
    getFilterNoItemsFoundText() {
        return 'No items found';
    }
    /**
     * @param {?} isCollapse
     * @return {?}
     */
    getTooltipCollapseExpandText(isCollapse) {
        return isCollapse ? 'Expand' : 'Collapse';
    }
}
TreeviewI18nDefault.decorators = [
    { type: Injectable }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const TreeviewHelper = {
    findItem,
    findItemInList,
    findParent,
    removeItem,
    concatSelection,
    findById,
    findRoot
};
/**
 * @param {?} root
 * @param {?} value
 * @return {?}
 */
function findItem(root, value) {
    if (isNil(root)) {
        return undefined;
    }
    if (root.value === value) {
        return root;
    }
    if (root.children) {
        for (const child of root.children) {
            /** @type {?} */
            const foundItem = findItem(child, value);
            if (foundItem) {
                return foundItem;
            }
        }
    }
    return undefined;
}
/**
 * @param {?} list
 * @param {?} value
 * @return {?}
 */
function findItemInList(list, value) {
    if (isNil(list)) {
        return undefined;
    }
    for (const item of list) {
        /** @type {?} */
        const foundItem = findItem(item, value);
        if (foundItem) {
            return foundItem;
        }
    }
    return undefined;
}
/**
 * @param {?} root
 * @param {?} item
 * @return {?}
 */
function findParent(root, item) {
    if (isNil(root) || isNil(root.children)) {
        return undefined;
    }
    for (const child of root.children) {
        if (child === item) {
            return root;
        }
        else {
            /** @type {?} */
            const parent = findParent(child, item);
            if (parent) {
                return parent;
            }
        }
    }
    return undefined;
}
/**
 * @param {?} root
 * @param {?} id
 * @return {?}
 */
function findById(root, id) {
    if (isNil(root)) {
        return undefined;
    }
    for (const child of root) {
        if (child.id === id) {
            return child;
        }
        else {
            if (child.children) {
                /** @type {?} */
                const parent = findById(child.children, id);
                if (parent) {
                    return parent;
                }
            }
        }
    }
    return undefined;
}
/**
 * @param {?} root
 * @param {?} item
 * @return {?}
 */
function removeItem(root, item) {
    /** @type {?} */
    const parent = findParent(root, item);
    if (parent) {
        pull(parent.children, item);
        if (parent.children.length === 0) {
            parent.children = undefined;
        }
        else {
            parent.correctChecked();
        }
        return true;
    }
    return false;
}
/**
 * @param {?} items
 * @param {?} checked
 * @param {?} unchecked
 * @return {?}
 */
function concatSelection(items, checked, unchecked) {
    /** @type {?} */
    let checkedItems = [...checked];
    /** @type {?} */
    let uncheckedItems = [...unchecked];
    for (const item of items) {
        /** @type {?} */
        const selection = item.getSelection();
        checkedItems = concat(checkedItems, selection.checkedItems);
        uncheckedItems = concat(uncheckedItems, selection.uncheckedItems);
    }
    return {
        checked: checkedItems,
        unchecked: uncheckedItems
    };
}
/**
 * @param {?} root
 * @param {?} id
 * @param {?=} isRoot
 * @param {?=} rootNode
 * @return {?}
 */
function findRoot(root, id, isRoot = true, rootNode = null) {
    if (isNil(root)) {
        return undefined;
    }
    for (const child of root) {
        rootNode = isRoot ? child : rootNode;
        if (child.id === id) {
            return rootNode;
        }
        else {
            if (child.children) {
                /** @type {?} */
                const parent = findRoot(child.children, id, false, rootNode);
                if (parent) {
                    return parent;
                }
            }
        }
    }
    return undefined;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function TreeviewSelection() { }
if (false) {
    /** @type {?} */
    TreeviewSelection.prototype.checkedItems;
    /** @type {?} */
    TreeviewSelection.prototype.uncheckedItems;
}
/**
 * @record
 */
function TreeItem() { }
if (false) {
    /** @type {?|undefined} */
    TreeItem.prototype.id;
    /** @type {?|undefined} */
    TreeItem.prototype.parentId;
    /** @type {?} */
    TreeItem.prototype.text;
    /** @type {?} */
    TreeItem.prototype.value;
    /** @type {?|undefined} */
    TreeItem.prototype.level;
    /** @type {?|undefined} */
    TreeItem.prototype.disabled;
    /** @type {?|undefined} */
    TreeItem.prototype.isLast;
    /** @type {?|undefined} */
    TreeItem.prototype.checked;
    /** @type {?|undefined} */
    TreeItem.prototype.collapsed;
    /** @type {?|undefined} */
    TreeItem.prototype.children;
}
class TreeviewItem {
    /**
     * @param {?} item
     * @param {?=} autoCorrectChecked
     */
    constructor(item, autoCorrectChecked = true) {
        this.internalDisabled = false;
        this.internalChecked = false;
        this.internalCollapsed = false;
        this.internalIsLast = true;
        if (isNil(item)) {
            throw new Error('Item must be defined');
        }
        if (isString(item.text)) {
            this.text = item.text;
        }
        else {
            throw new Error('A text of item must be string object');
        }
        this.id = item.id;
        this.parentId = item.parentId;
        this.value = item.value;
        this.level = item.level;
        if (isBoolean(item.checked)) {
            this.checked = item.checked;
        }
        if (isBoolean(item.collapsed)) {
            this.collapsed = item.collapsed;
        }
        if (isBoolean(item.disabled)) {
            this.disabled = item.disabled;
        }
        if (isBoolean(item.isLast)) {
            this.isLast = item.isLast;
        }
        if (!isNil(item.children) && item.children.length > 0) {
            this.children = item.children.map((/**
             * @param {?} child
             * @return {?}
             */
            child => {
                if (this.disabled === true) {
                    child.disabled = true;
                }
                return new TreeviewItem(child);
            }));
        }
        if (autoCorrectChecked) {
            this.correctChecked();
        }
    }
    /**
     * @return {?}
     */
    get checked() {
        return this.internalChecked;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set checked(value) {
        if (!this.internalDisabled) {
            if (this.internalChecked !== value) {
                this.internalChecked = value;
            }
        }
    }
    /**
     * @return {?}
     */
    get isLast() {
        return this.internalIsLast;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set isLast(value) {
        this.internalIsLast = value;
    }
    /**
     * @return {?}
     */
    get indeterminate() {
        return this.checked === undefined;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    setCheckedRecursive(value) {
        if (!this.internalDisabled) {
            this.internalChecked = value;
            if (!isNil(this.internalChildren)) {
                this.internalChildren.forEach((/**
                 * @param {?} child
                 * @return {?}
                 */
                child => child.setCheckedRecursive(value)));
            }
        }
    }
    /**
     * @return {?}
     */
    get disabled() {
        return this.internalDisabled;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set disabled(value) {
        if (this.internalDisabled !== value) {
            this.internalDisabled = value;
            if (!isNil(this.internalChildren)) {
                this.internalChildren.forEach((/**
                 * @param {?} child
                 * @return {?}
                 */
                child => child.disabled = value));
            }
        }
    }
    /**
     * @return {?}
     */
    get collapsed() {
        return this.internalCollapsed;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set collapsed(value) {
        if (this.internalCollapsed !== value) {
            this.internalCollapsed = value;
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    setCollapsedRecursive(value) {
        this.internalCollapsed = value;
        if (!isNil(this.internalChildren)) {
            this.internalChildren.forEach((/**
             * @param {?} child
             * @return {?}
             */
            child => child.setCollapsedRecursive(value)));
        }
    }
    /**
     * @return {?}
     */
    get children() {
        return this.internalChildren;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set children(value) {
        if (this.internalChildren !== value) {
            if (!isNil(value) && value.length === 0) {
                throw new Error('Children must be not an empty array');
            }
            this.internalChildren = value;
            if (!isNil(this.internalChildren)) {
                /** @type {?} */
                let checked = null;
                this.internalChildren.forEach((/**
                 * @param {?} child
                 * @return {?}
                 */
                child => {
                    if (checked === null) {
                        checked = child.checked;
                    }
                    else {
                        if (child.checked !== checked) {
                            checked = undefined;
                            return;
                        }
                    }
                }));
                this.internalChecked = checked;
            }
        }
    }
    /**
     * @return {?}
     */
    getSelection() {
        /** @type {?} */
        let checkedItems = [];
        /** @type {?} */
        let uncheckedItems = [];
        if (isNil(this.internalChildren)) {
            if (this.internalChecked) {
                checkedItems.push(this);
            }
            else {
                uncheckedItems.push(this);
            }
        }
        else {
            /** @type {?} */
            const selection = TreeviewHelper.concatSelection(this.internalChildren, checkedItems, uncheckedItems);
            checkedItems = selection.checked;
            uncheckedItems = selection.unchecked;
        }
        return {
            checkedItems,
            uncheckedItems
        };
    }
    /**
     * @return {?}
     */
    correctChecked() {
        this.internalChecked = this.getCorrectChecked();
    }
    /**
     * @private
     * @return {?}
     */
    getCorrectChecked() {
        /** @type {?} */
        let checked = null;
        if (!isNil(this.internalChildren)) {
            for (const child of this.internalChildren) {
                child.internalChecked = child.getCorrectChecked();
                if (checked === null) {
                    checked = child.internalChecked;
                }
                else if (checked !== child.internalChecked) {
                    checked = undefined;
                    break;
                }
            }
        }
        else {
            checked = this.checked;
        }
        return checked;
    }
}
if (false) {
    /**
     * @type {?}
     * @private
     */
    TreeviewItem.prototype.internalDisabled;
    /**
     * @type {?}
     * @private
     */
    TreeviewItem.prototype.internalChecked;
    /**
     * @type {?}
     * @private
     */
    TreeviewItem.prototype.internalCollapsed;
    /**
     * @type {?}
     * @private
     */
    TreeviewItem.prototype.internalIsLast;
    /**
     * @type {?}
     * @private
     */
    TreeviewItem.prototype.internalChildren;
    /** @type {?} */
    TreeviewItem.prototype.id;
    /** @type {?} */
    TreeviewItem.prototype.parentId;
    /** @type {?} */
    TreeviewItem.prototype.text;
    /** @type {?} */
    TreeviewItem.prototype.value;
    /** @type {?} */
    TreeviewItem.prototype.level;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TreeviewConfig {
    constructor() {
        this.hasAllCheckBox = true;
        this.hasFilter = false;
        this.hasCollapseExpand = false;
        this.decoupleChildFromParent = false;
        this.maxHeight = 500;
    }
    /**
     * @return {?}
     */
    get hasDivider() {
        return this.hasFilter || this.hasAllCheckBox || this.hasCollapseExpand;
    }
    /**
     * @param {?=} fields
     * @return {?}
     */
    static create(fields) {
        /** @type {?} */
        const config = new TreeviewConfig();
        Object.assign(config, fields);
        return config;
    }
}
TreeviewConfig.decorators = [
    { type: Injectable }
];
if (false) {
    /** @type {?} */
    TreeviewConfig.prototype.hasAllCheckBox;
    /** @type {?} */
    TreeviewConfig.prototype.hasFilter;
    /** @type {?} */
    TreeviewConfig.prototype.hasCollapseExpand;
    /** @type {?} */
    TreeviewConfig.prototype.decoupleChildFromParent;
    /** @type {?} */
    TreeviewConfig.prototype.maxHeight;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
class TreeviewEventParser {
}
TreeviewEventParser.decorators = [
    { type: Injectable }
];
if (false) {
    /**
     * @abstract
     * @param {?} component
     * @return {?}
     */
    TreeviewEventParser.prototype.getSelectedChange = function (component) { };
}
class DefaultTreeviewEventParser extends TreeviewEventParser {
    /**
     * @param {?} component
     * @return {?}
     */
    getSelectedChange(component) {
        /** @type {?} */
        const checkedItems = component.selection.checkedItems;
        if (!isNil(checkedItems)) {
            return checkedItems.map((/**
             * @param {?} item
             * @return {?}
             */
            item => item.value));
        }
        return [];
    }
}
DefaultTreeviewEventParser.decorators = [
    { type: Injectable }
];
/**
 * @record
 */
function DownlineTreeviewItem() { }
if (false) {
    /** @type {?} */
    DownlineTreeviewItem.prototype.item;
    /** @type {?} */
    DownlineTreeviewItem.prototype.parent;
}
class DownlineTreeviewEventParser extends TreeviewEventParser {
    /**
     * @param {?} component
     * @return {?}
     */
    getSelectedChange(component) {
        /** @type {?} */
        const items = component.items;
        if (!isNil(items)) {
            /** @type {?} */
            let result = [];
            items.forEach((/**
             * @param {?} item
             * @return {?}
             */
            item => {
                /** @type {?} */
                const links = this.getLinks(item, null);
                if (!isNil(links)) {
                    result = result.concat(links);
                }
            }));
            return result;
        }
        return [];
    }
    /**
     * @private
     * @param {?} item
     * @param {?} parent
     * @return {?}
     */
    getLinks(item, parent) {
        if (!isNil(item.children)) {
            /** @type {?} */
            const link = {
                item,
                parent
            };
            /** @type {?} */
            let result = [];
            item.children.forEach((/**
             * @param {?} child
             * @return {?}
             */
            child => {
                /** @type {?} */
                const links = this.getLinks(child, link);
                if (!isNil(links)) {
                    result = result.concat(links);
                }
            }));
            return result;
        }
        if (item.checked) {
            return [{
                    item,
                    parent
                }];
        }
        return null;
    }
}
DownlineTreeviewEventParser.decorators = [
    { type: Injectable }
];
class OrderDownlineTreeviewEventParser extends TreeviewEventParser {
    constructor() {
        super(...arguments);
        this.currentDownlines = [];
        this.parser = new DownlineTreeviewEventParser();
    }
    /**
     * @param {?} component
     * @return {?}
     */
    getSelectedChange(component) {
        /** @type {?} */
        const newDownlines = this.parser.getSelectedChange(component);
        if (this.currentDownlines.length === 0) {
            this.currentDownlines = newDownlines;
        }
        else {
            /** @type {?} */
            const intersectDownlines = [];
            this.currentDownlines.forEach((/**
             * @param {?} downline
             * @return {?}
             */
            downline => {
                /** @type {?} */
                let foundIndex = -1;
                /** @type {?} */
                const length = newDownlines.length;
                for (let i = 0; i < length; i++) {
                    if (downline.item.value === newDownlines[i].item.value) {
                        foundIndex = i;
                        break;
                    }
                }
                if (foundIndex !== -1) {
                    intersectDownlines.push(newDownlines[foundIndex]);
                    newDownlines.splice(foundIndex, 1);
                }
            }));
            this.currentDownlines = intersectDownlines.concat(newDownlines);
        }
        return this.currentDownlines;
    }
}
OrderDownlineTreeviewEventParser.decorators = [
    { type: Injectable }
];
if (false) {
    /**
     * @type {?}
     * @private
     */
    OrderDownlineTreeviewEventParser.prototype.currentDownlines;
    /**
     * @type {?}
     * @private
     */
    OrderDownlineTreeviewEventParser.prototype.parser;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TreeviewItemComponent {
    /**
     * @param {?} defaultConfig
     */
    constructor(defaultConfig) {
        this.defaultConfig = defaultConfig;
        this.checkedChange = new EventEmitter();
        this.onCollapseExpand = (/**
         * @return {?}
         */
        () => {
            this.item.collapsed = !this.item.collapsed;
        });
        this.onCheckedChange = (/**
         * @return {?}
         */
        () => {
            /** @type {?} */
            const checked = this.item.checked;
            if (!isNil(this.item.children) && !this.config.decoupleChildFromParent) {
                this.item.children.forEach((/**
                 * @param {?} child
                 * @return {?}
                 */
                child => child.setCheckedRecursive(checked)));
            }
            this.checkedChange.emit({
                item: this.item,
                checked
            });
        });
        this.config = this.defaultConfig;
    }
    /**
     * @param {?} item
     * @return {?}
     */
    triggerItemChange(item) {
        this.checkedChange.emit({
            item,
            checked: item.checked
        });
        // make sure that the parent correct checked
        this.item.correctChecked();
    }
    /**
     * @param {?} item
     * @return {?}
     */
    onLabelClick(item) {
        // Check last group
        // if (!item.isLast) {
        //     return;
        // }
        item.checked = !item.checked;
        this.onCheckedChange();
    }
    /**
     * @param {?} root
     * @param {?} child
     * @return {?}
     */
    onChildCheckedChange(root, child) {
        if (!this.config.decoupleChildFromParent) {
            /** @type {?} */
            let itemChecked = null;
            for (const childItem of this.item.children) {
                if (itemChecked === null) {
                    itemChecked = childItem.checked;
                }
                else if (itemChecked !== childItem.checked) {
                    itemChecked = undefined;
                    break;
                }
            }
            if (itemChecked === null) {
                itemChecked = false;
            }
            if (this.item.checked !== itemChecked) {
                this.item.checked = itemChecked;
            }
        }
        this.checkedChange.emit({
            item: child.item,
            checked: child.checked
        });
    }
}
TreeviewItemComponent.decorators = [
    { type: Component, args: [{
                selector: 'rs-dropdown-tree-item',
                template: "<div *ngIf=\"item\" class=\"treeview-item\">\n    <ng-template [ngTemplateOutlet]=\"template\"\n        [ngTemplateOutletContext]=\"{item: item, onCollapseExpand: onCollapseExpand, onCheckedChange: onCheckedChange, onLabelClick: onLabelClick}\">\n    </ng-template>\n    <div *ngIf=\"!item.collapsed\">\n        <rs-dropdown-tree-item [config]=\"config\" *ngFor=\"let child of item.children\" [item]=\"child\"\n            [template]=\"template\" (checkedChange)=\"onChildCheckedChange(child, $event)\">\n        </rs-dropdown-tree-item>\n    </div>\n</div>",
                styles: [":host{display:block}:host .treeview-item{white-space:nowrap}:host .treeview-item .treeview-item{margin-left:2rem}"]
            }] }
];
/** @nocollapse */
TreeviewItemComponent.ctorParameters = () => [
    { type: TreeviewConfig }
];
TreeviewItemComponent.propDecorators = {
    config: [{ type: Input }],
    template: [{ type: Input }],
    item: [{ type: Input }],
    checkedChange: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    TreeviewItemComponent.prototype.config;
    /** @type {?} */
    TreeviewItemComponent.prototype.template;
    /** @type {?} */
    TreeviewItemComponent.prototype.item;
    /** @type {?} */
    TreeviewItemComponent.prototype.checkedChange;
    /** @type {?} */
    TreeviewItemComponent.prototype.onCollapseExpand;
    /** @type {?} */
    TreeviewItemComponent.prototype.onCheckedChange;
    /**
     * @type {?}
     * @private
     */
    TreeviewItemComponent.prototype.defaultConfig;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FilterTreeviewItem extends TreeviewItem {
    /**
     * @param {?} item
     */
    constructor(item) {
        super({
            id: item.id,
            parentId: item.parentId,
            text: item.text,
            value: item.value,
            disabled: item.disabled,
            checked: item.checked,
            collapsed: item.collapsed,
            children: item.children
        });
        this.refItem = item;
    }
    /**
     * @return {?}
     */
    updateRefChecked() {
        this.children.forEach((/**
         * @param {?} child
         * @return {?}
         */
        child => {
            if (child instanceof FilterTreeviewItem) {
                child.updateRefChecked();
            }
        }));
        /** @type {?} */
        let refChecked = this.checked;
        if (refChecked) {
            for (const refChild of this.refItem.children) {
                if (!refChild.checked) {
                    refChecked = false;
                    break;
                }
            }
        }
        this.refItem.checked = refChecked;
    }
}
if (false) {
    /**
     * @type {?}
     * @private
     */
    FilterTreeviewItem.prototype.refItem;
}
class TreeviewComponent {
    /**
     * @param {?} i18n
     * @param {?} defaultConfig
     * @param {?} eventParser
     */
    constructor(i18n, defaultConfig, eventParser) {
        this.i18n = i18n;
        this.defaultConfig = defaultConfig;
        this.eventParser = eventParser;
        this.items = [];
        this.selectedChange = new EventEmitter();
        this.filterChange = new EventEmitter();
        this.filterText = '';
        this.text$ = new Subject();
        this.config = this.defaultConfig;
        this.allItem = new TreeviewItem({ text: 'All', value: undefined, id: undefined, collapsed: false });
        this.createHeaderTemplateContext();
        this.text$.pipe(debounceTime(500), distinctUntilChanged())
            .subscribe((/**
         * @return {?}
         */
        () => {
            this.filterChange.emit(this.filterText);
            this.updateFilterItems();
        }));
    }
    /**
     * @return {?}
     */
    get hasFilterItems() {
        return !isNil(this.filterItems) && this.filterItems.length > 0;
    }
    /**
     * @return {?}
     */
    get maxHeight() {
        return `${this.config.maxHeight}`;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        /** @type {?} */
        const itemsSimpleChange = changes.items;
        if (!isNil(itemsSimpleChange)) {
            if (!isNil(this.items)) {
                this.updateFilterItems();
                this.updateCollapsedOfAll();
                this.raiseSelectedChange();
            }
        }
        this.createHeaderTemplateContext();
    }
    /**
     * @return {?}
     */
    onAllCollapseExpand() {
        this.allItem.collapsed = !this.allItem.collapsed;
        this.filterItems.forEach((/**
         * @param {?} item
         * @return {?}
         */
        item => item.setCollapsedRecursive(this.allItem.collapsed)));
    }
    /**
     * @param {?} text
     * @return {?}
     */
    onFilterTextChange(text) {
        this.filterText = text;
        // debounce text change by 500ms
        this.text$.next(text);
    }
    /**
     * @return {?}
     */
    onAllCheckedChange() {
        /** @type {?} */
        const checked = this.allItem.checked;
        this.filterItems.forEach((/**
         * @param {?} item
         * @return {?}
         */
        item => {
            item.setCheckedRecursive(checked);
            if (item instanceof FilterTreeviewItem) {
                item.updateRefChecked();
            }
        }));
        this.raiseSelectedChange();
    }
    /**
     * @param {?} item
     * @param {?} child
     * @return {?}
     */
    onItemCheckedChange(item, child) {
        if (child.checked === undefined) {
            return;
        }
        if (item instanceof FilterTreeviewItem) {
            item.updateRefChecked();
        }
        // Select ONLY last group
        // if (child.checked && child.item) {
        //     this.deselectCheckedFromOther(child.item);
        // }
        this.updateCheckedOfAll();
        this.raiseSelectedChange();
    }
    /**
     * @param {?} item
     * @return {?}
     */
    deselectCheckedFromOther(item) {
        /** @type {?} */
        const selected = this.selection.checkedItems;
        if (selected.length) {
            /** @type {?} */
            const isParent = item.children && item.children.length > 0;
            if (selected[0].parentId !== item.parentId || isParent) {
                // Get parent of selected
                /** @type {?} */
                const found = TreeviewHelper.findById(this.items, selected[0].parentId);
                if (found) {
                    // Uncheck all selected
                    found.setCheckedRecursive(false);
                    if (isParent) {
                        found.checked = undefined;
                        item.setCheckedRecursive(true); // Check current selected
                    }
                    else {
                        /** @type {?} */
                        const foundParentSelected = TreeviewHelper.findById(this.items, item.parentId);
                        if (foundParentSelected) {
                            foundParentSelected.checked = undefined;
                        }
                        item.checked = true;
                    }
                }
                this.items.forEach((/**
                 * @param {?} node
                 * @return {?}
                 */
                (node) => {
                    node.correctChecked();
                }));
            }
        }
    }
    /**
     * @return {?}
     */
    raiseSelectedChange() {
        this.generateSelection();
        /** @type {?} */
        const values = this.eventParser.getSelectedChange(this);
        this.selectedChange.emit(values);
    }
    /**
     * @private
     * @return {?}
     */
    createHeaderTemplateContext() {
        this.headerTemplateContext = {
            config: this.config,
            item: this.allItem,
            onCheckedChange: (/**
             * @return {?}
             */
            () => this.onAllCheckedChange()),
            onCollapseExpand: (/**
             * @return {?}
             */
            () => this.onAllCollapseExpand()),
            onFilterTextChange: (/**
             * @param {?} text
             * @return {?}
             */
            (text) => this.onFilterTextChange(text))
        };
    }
    /**
     * @private
     * @return {?}
     */
    generateSelection() {
        /** @type {?} */
        let checkedItems = [];
        /** @type {?} */
        let uncheckedItems = [];
        if (!isNil(this.items)) {
            /** @type {?} */
            const selection = TreeviewHelper.concatSelection(this.items, checkedItems, uncheckedItems);
            checkedItems = selection.checked;
            uncheckedItems = selection.unchecked;
        }
        this.selection = {
            checkedItems,
            uncheckedItems
        };
    }
    /**
     * @private
     * @return {?}
     */
    updateFilterItems() {
        if (this.filterText !== '') {
            /** @type {?} */
            const filterItems = [];
            /** @type {?} */
            const filterText = this.filterText.toLowerCase();
            this.items.forEach((/**
             * @param {?} item
             * @return {?}
             */
            item => {
                /** @type {?} */
                const newItem = this.filterItem(item, filterText);
                if (!isNil(newItem)) {
                    filterItems.push(newItem);
                }
            }));
            this.filterItems = filterItems;
        }
        else {
            this.filterItems = this.items;
        }
        this.updateCheckedOfAll();
    }
    /**
     * @private
     * @param {?} item
     * @param {?} filterText
     * @return {?}
     */
    filterItem(item, filterText) {
        /** @type {?} */
        const isMatch = includes(item.text.toLowerCase(), filterText);
        if (isMatch) {
            return item;
        }
        else {
            if (!isNil(item.children)) {
                /** @type {?} */
                const children = [];
                item.children.forEach((/**
                 * @param {?} child
                 * @return {?}
                 */
                child => {
                    /** @type {?} */
                    const newChild = this.filterItem(child, filterText);
                    if (!isNil(newChild)) {
                        children.push(newChild);
                    }
                }));
                if (children.length > 0) {
                    /** @type {?} */
                    const newItem = new FilterTreeviewItem(item);
                    newItem.collapsed = false;
                    newItem.children = children;
                    return newItem;
                }
            }
        }
        return undefined;
    }
    /**
     * @private
     * @return {?}
     */
    updateCheckedOfAll() {
        /** @type {?} */
        let itemChecked = null;
        for (const filterItem of this.filterItems) {
            if (itemChecked === null) {
                itemChecked = filterItem.checked;
            }
            else if (itemChecked !== filterItem.checked) {
                itemChecked = undefined;
                break;
            }
        }
        if (itemChecked === null) {
            itemChecked = false;
        }
        this.allItem.checked = itemChecked;
    }
    /**
     * @private
     * @return {?}
     */
    updateCollapsedOfAll() {
        /** @type {?} */
        let hasItemExpanded = false;
        for (const filterItem of this.filterItems) {
            if (!filterItem.collapsed) {
                hasItemExpanded = true;
                break;
            }
        }
        this.allItem.collapsed = !hasItemExpanded;
    }
}
TreeviewComponent.decorators = [
    { type: Component, args: [{
                selector: 'rs-dropdown-tree',
                template: "<ng-template #defaultItemTemplate let-item=\"item\" let-onCollapseExpand=\"onCollapseExpand\"\n  let-onCheckedChange=\"onCheckedChange\">\n  <div class=\"form-inline row-item\">\n    <i *ngIf=\"item.children\" (click)=\"onCollapseExpand()\" aria-hidden=\"true\" class=\"fa\"\n      [class.fa-caret-right]=\"item.collapsed\" [class.fa-caret-down]=\"!item.collapsed\"></i>\n    <div class=\"form-check\">\n      <input type=\"checkbox\" class=\"form-check-input\" [(ngModel)]=\"item.checked\" (ngModelChange)=\"onCheckedChange()\"\n        [disabled]=\"item.disabled\" [indeterminate]=\"item.indeterminate\" />\n      <label class=\"form-check-label\" (click)=\"item.checked = !item.checked; onCheckedChange()\">\n        {{item.text}}\n      </label>\n    </div>\n  </div>\n</ng-template>\n<ng-template #defaultHeaderTemplate let-config=\"config\" let-item=\"item\" let-onCollapseExpand=\"onCollapseExpand\"\n  let-onCheckedChange=\"onCheckedChange\" let-onFilterTextChange=\"onFilterTextChange\">\n  <div *ngIf=\"config.hasFilter\" class=\"row row-filter\">\n    <div class=\"col-12\">\n      <input class=\"form-control\" type=\"text\" [placeholder]=\"i18n.getFilterPlaceholder()\" [(ngModel)]=\"filterText\"\n        (ngModelChange)=\"onFilterTextChange($event)\" />\n    </div>\n  </div>\n  <div *ngIf=\"hasFilterItems\">\n    <div *ngIf=\"config.hasAllCheckBox || config.hasCollapseExpand\" class=\"row row-all\">\n      <div class=\"col-12\">\n        <div class=\"form-check form-check-inline\" *ngIf=\"config.hasAllCheckBox\">\n          <input type=\"checkbox\" class=\"form-check-input\" [(ngModel)]=\"item.checked\" (ngModelChange)=\"onCheckedChange()\"\n            [indeterminate]=\"item.indeterminate\" />\n          <label class=\"form-check-label\" (click)=\"item.checked = !item.checked; onCheckedChange()\">\n            {{i18n.getAllCheckboxText()}}\n          </label>\n        </div>\n        <label *ngIf=\"config.hasCollapseExpand\" class=\"form-check-label\" (click)=\"onCollapseExpand()\">\n          <i [title]=\"i18n.getTooltipCollapseExpandText(item.collapsed)\" aria-hidden=\"true\" class=\"fa\"\n            [class.fa-expand]=\"item.collapsed\" [class.fa-compress]=\"!item.collapsed\" [style.marginLeft]=\"'-10px'\"></i>\n        </label>\n      </div>\n    </div>\n    <div *ngIf=\"config.hasDivider\" class=\"dropdown-divider\"></div>\n  </div>\n</ng-template>\n<div class=\"treeview-header\">\n  <ng-template [ngTemplateOutlet]=\"headerTemplate || defaultHeaderTemplate\"\n    [ngTemplateOutletContext]=\"headerTemplateContext\">\n  </ng-template>\n</div>\n<div [ngSwitch]=\"hasFilterItems\">\n  <div *ngSwitchCase=\"true\" class=\"treeview-container\" [style.max-height.px]=\"maxHeight\">\n    <rs-dropdown-tree-item *ngFor=\"let item of filterItems\" [config]=\"config\" [item]=\"item\"\n      [template]=\"itemTemplate || defaultItemTemplate\" (checkedChange)=\"onItemCheckedChange(item, $event)\">\n    </rs-dropdown-tree-item>\n  </div>\n  <div *ngSwitchCase=\"false\" class=\"treeview-text\">\n    {{i18n.getFilterNoItemsFoundText()}}\n  </div>\n</div>",
                styles: [":host::deep .treeview-header .row-filter{margin-bottom:.5rem}:host::deep .treeview-header .row-all .fa{cursor:pointer}:host::deep .treeview-container .row-item{margin-bottom:.3rem;flex-wrap:nowrap}:host::deep .treeview-container .row-item .fa{width:.8rem;cursor:pointer;margin-right:.3rem}.treeview-container{overflow-y:auto;padding-right:.3rem}.treeview-text{padding:.3rem 0;white-space:nowrap}.form-check-label{position:absolute;top:-30px;right:26px;cursor:pointer}"]
            }] }
];
/** @nocollapse */
TreeviewComponent.ctorParameters = () => [
    { type: TreeviewI18n },
    { type: TreeviewConfig },
    { type: TreeviewEventParser }
];
TreeviewComponent.propDecorators = {
    headerTemplate: [{ type: Input }],
    itemTemplate: [{ type: Input }],
    items: [{ type: Input }],
    config: [{ type: Input }],
    selectedChange: [{ type: Output }],
    filterChange: [{ type: Output }],
    treeviewItemComponent: [{ type: ViewChild, args: [TreeviewItemComponent, { static: false },] }]
};
if (false) {
    /** @type {?} */
    TreeviewComponent.prototype.headerTemplate;
    /** @type {?} */
    TreeviewComponent.prototype.itemTemplate;
    /** @type {?} */
    TreeviewComponent.prototype.items;
    /** @type {?} */
    TreeviewComponent.prototype.config;
    /** @type {?} */
    TreeviewComponent.prototype.selectedChange;
    /** @type {?} */
    TreeviewComponent.prototype.filterChange;
    /** @type {?} */
    TreeviewComponent.prototype.headerTemplateContext;
    /** @type {?} */
    TreeviewComponent.prototype.treeviewItemComponent;
    /** @type {?} */
    TreeviewComponent.prototype.allItem;
    /** @type {?} */
    TreeviewComponent.prototype.filterText;
    /** @type {?} */
    TreeviewComponent.prototype.filterItems;
    /** @type {?} */
    TreeviewComponent.prototype.selection;
    /** @type {?} */
    TreeviewComponent.prototype.text$;
    /** @type {?} */
    TreeviewComponent.prototype.i18n;
    /**
     * @type {?}
     * @private
     */
    TreeviewComponent.prototype.defaultConfig;
    /**
     * @type {?}
     * @private
     */
    TreeviewComponent.prototype.eventParser;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DropdownDirective {
    constructor() {
        // tslint:disable-next-line:no-input-rename
        this.internalOpen = false;
        this.openChange = new EventEmitter();
    }
    /**
     * @return {?}
     */
    get isOpen() {
        return this.internalOpen;
    }
    /**
     * @return {?}
     */
    onKeyupEsc() {
        this.close();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDocumentClick(event) {
        if (event.button !== 2 && !this.isEventFromToggle(event)) {
            this.close();
        }
    }
    /**
     * @return {?}
     */
    open() {
        if (!this.internalOpen) {
            this.internalOpen = true;
            this.openChange.emit(true);
        }
    }
    /**
     * @return {?}
     */
    close() {
        if (this.internalOpen) {
            this.internalOpen = false;
            this.openChange.emit(false);
        }
    }
    /**
     * @return {?}
     */
    toggle() {
        if (this.isOpen) {
            this.close();
        }
        else {
            this.open();
        }
    }
    /**
     * @private
     * @param {?} event
     * @return {?}
     */
    isEventFromToggle(event) {
        return !isNil(this.toggleElement) && this.toggleElement.contains(event.target);
    }
}
DropdownDirective.decorators = [
    { type: Directive, args: [{
                selector: '[rsDropdown]',
                exportAs: 'rsDropdown'
            },] }
];
DropdownDirective.propDecorators = {
    internalOpen: [{ type: Input, args: ['open',] }],
    openChange: [{ type: Output }],
    isOpen: [{ type: HostBinding, args: ['class.show',] }],
    onKeyupEsc: [{ type: HostListener, args: ['keyup.esc',] }],
    onDocumentClick: [{ type: HostListener, args: ['document:click', ['$event'],] }]
};
if (false) {
    /** @type {?} */
    DropdownDirective.prototype.toggleElement;
    /** @type {?} */
    DropdownDirective.prototype.internalOpen;
    /** @type {?} */
    DropdownDirective.prototype.openChange;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DropdownMenuDirective {
    /**
     * @param {?} dropdown
     */
    constructor(dropdown) {
        this.dropdown = dropdown;
        this.dropdownMenu = true;
    }
    /**
     * @return {?}
     */
    get isOpen() {
        return this.dropdown.isOpen;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        console.log(changes);
    }
}
DropdownMenuDirective.decorators = [
    { type: Directive, args: [{
                selector: '[rsDropdownMenu]'
            },] }
];
/** @nocollapse */
DropdownMenuDirective.ctorParameters = () => [
    { type: DropdownDirective }
];
DropdownMenuDirective.propDecorators = {
    dropdownMenu: [{ type: HostBinding, args: ['class.dropdown-menu',] }],
    isOpen: [{ type: HostBinding, args: ['class.show',] }]
};
if (false) {
    /** @type {?} */
    DropdownMenuDirective.prototype.dropdownMenu;
    /** @type {?} */
    DropdownMenuDirective.prototype.dropdown;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DropdownToggleDirective {
    /**
     * @param {?} dropdown
     * @param {?} elementRef
     */
    constructor(dropdown, elementRef) {
        this.dropdown = dropdown;
        this.classes = true;
        this.hasPopup = true;
        this.expanded = 'dropdown.isOpen';
        dropdown.toggleElement = elementRef.nativeElement;
    }
    /**
     * @return {?}
     */
    onClick() {
        this.dropdown.toggle();
    }
}
DropdownToggleDirective.decorators = [
    { type: Directive, args: [{
                selector: '[rsDropdownToggle]'
            },] }
];
/** @nocollapse */
DropdownToggleDirective.ctorParameters = () => [
    { type: DropdownDirective },
    { type: ElementRef }
];
DropdownToggleDirective.propDecorators = {
    classes: [{ type: HostBinding, args: ['class.dropdown-toggle',] }],
    hasPopup: [{ type: HostBinding, args: ['attr.aria-haspopup',] }],
    expanded: [{ type: HostBinding, args: ['attr.aria-expanded',] }],
    onClick: [{ type: HostListener, args: ['click',] }]
};
if (false) {
    /** @type {?} */
    DropdownToggleDirective.prototype.classes;
    /** @type {?} */
    DropdownToggleDirective.prototype.hasPopup;
    /** @type {?} */
    DropdownToggleDirective.prototype.expanded;
    /** @type {?} */
    DropdownToggleDirective.prototype.dropdown;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DropdownTreeviewComponent {
    /**
     * @param {?} i18n
     * @param {?} defaultConfig
     */
    constructor(i18n, defaultConfig) {
        this.i18n = i18n;
        this.defaultConfig = defaultConfig;
        this.buttonClass = 'btn-outline-secondary';
        this.items = [];
        this.selectedChange = new EventEmitter(true);
        this.filterChange = new EventEmitter();
        this.config = this.defaultConfig;
        this.config.decoupleChildFromParent = true;
    }
    /**
     * @return {?}
     */
    getText() {
        return this.i18n.getText(this.treeviewComponent.selection);
    }
    /**
     * @return {?}
     */
    getSelection() {
        if (!this.treeviewComponent || !this.treeviewComponent.selection || !this.treeviewComponent.selection.checkedItems) {
            return [];
        }
        return this.treeviewComponent.selection.checkedItems;
    }
    /**
     * @param {?} event
     * @param {?} item
     * @return {?}
     */
    removeItem(event, item) {
        event.stopImmediatePropagation();
        item.checked = false;
        this.treeviewComponent.treeviewItemComponent.triggerItemChange(item);
        if (this.treeviewComponent.selection.checkedItems.length === 0) {
            this.treeviewComponent.allItem.checked = false;
            this.treeviewComponent.onAllCheckedChange();
        }
        /** @type {?} */
        const found = TreeviewHelper.findRoot(this.items, item.parentId);
        if (found) {
            found.correctChecked();
        }
    }
    /**
     * @param {?} values
     * @return {?}
     */
    onSelectedChange(values) {
        this.selectedChange.emit(values);
    }
    /**
     * @param {?} text
     * @return {?}
     */
    onFilterChange(text) {
        this.filterChange.emit(text);
    }
}
DropdownTreeviewComponent.decorators = [
    { type: Component, args: [{
                selector: 'rs-dropdown-treeview',
                template: "<div class=\"dropdown\" rsDropdown>\n  <!-- <button class=\"btn\" [ngClass]=\"buttonClass\" type=\"button\" role=\"button\" rsDropdownToggle>\n    {{getText()}}\n  </button> -->\n  <button class=\"btn\" [ngClass]=\"buttonClass\" type=\"button\" role=\"button\" rsDropdownToggle>\n    <span *ngIf=\"getSelection()?.length <= 0\">-- Select --</span>\n\n    <span *ngFor=\"let itm of getSelection() | slice:0:5; let idx=index\">\n      <span class=\"btn btn-default btn-xs pull-left border sel-item\">\n        {{itm.text}}\n        <a class=\"close pull-right\" (click)=\"removeItem($event, itm)\">&times;</a>\n      </span>\n    </span>\n\n    <span *ngIf=\"getSelection()?.length > 5\" class=\"pull-right\">{{getSelection().length - 5}} more...</span>\n  </button>\n  <div rsDropdownMenu aria-labelledby=\"dropdownMenu\" (click)=\"$event.stopPropagation()\">\n    <div class=\"dropdown-container\">\n      <rs-dropdown-tree [config]=\"config\" [headerTemplate]=\"headerTemplate\" [items]=\"items\"\n        [itemTemplate]=\"itemTemplate\" (selectedChange)=\"onSelectedChange($event)\"\n        (filterChange)=\"onFilterChange($event)\">\n      </rs-dropdown-tree>\n    </div>\n  </div>\n</div>",
                styles: [".dropdown{width:100%;display:inline-block}.dropdown button{width:100%;margin-right:.9rem;text-align:left;overflow:hidden;text-overflow:ellipsis;padding:3px}.dropdown button::after{position:absolute;right:.6rem;margin-top:.6rem}.dropdown button .btn.btn-default:hover{color:inherit!important;background:inherit!important;border-color:inherit!important}.dropdown .sel-item{width:-webkit-max-content;width:-moz-max-content;width:max-content;height:25px;font-size:12px;line-height:1;margin:3px;padding:4px 6px}.dropdown .sel-item .close{margin-left:6px;padding-top:2px;float:right;font-size:.8rem;line-height:1;border-radius:100%;color:#fff;background-color:#000;width:16px;height:16px}.dropdown .dropdown-menu{width:100%;z-index:9}.dropdown .dropdown-menu .dropdown-container{padding:0 .6rem}"]
            }] }
];
/** @nocollapse */
DropdownTreeviewComponent.ctorParameters = () => [
    { type: TreeviewI18n },
    { type: TreeviewConfig }
];
DropdownTreeviewComponent.propDecorators = {
    buttonClass: [{ type: Input }],
    headerTemplate: [{ type: Input }],
    itemTemplate: [{ type: Input }],
    items: [{ type: Input }],
    config: [{ type: Input }],
    selectedChange: [{ type: Output }],
    filterChange: [{ type: Output }],
    treeviewComponent: [{ type: ViewChild, args: [TreeviewComponent, { static: true },] }],
    dropdownDirective: [{ type: ViewChild, args: [DropdownDirective, { static: true },] }]
};
if (false) {
    /** @type {?} */
    DropdownTreeviewComponent.prototype.buttonClass;
    /** @type {?} */
    DropdownTreeviewComponent.prototype.headerTemplate;
    /** @type {?} */
    DropdownTreeviewComponent.prototype.itemTemplate;
    /** @type {?} */
    DropdownTreeviewComponent.prototype.items;
    /** @type {?} */
    DropdownTreeviewComponent.prototype.config;
    /** @type {?} */
    DropdownTreeviewComponent.prototype.selectedChange;
    /** @type {?} */
    DropdownTreeviewComponent.prototype.filterChange;
    /** @type {?} */
    DropdownTreeviewComponent.prototype.treeviewComponent;
    /** @type {?} */
    DropdownTreeviewComponent.prototype.dropdownDirective;
    /** @type {?} */
    DropdownTreeviewComponent.prototype.i18n;
    /**
     * @type {?}
     * @private
     */
    DropdownTreeviewComponent.prototype.defaultConfig;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TreeviewPipe {
    /**
     * @param {?} objects
     * @param {?} textField
     * @return {?}
     */
    transform(objects, textField) {
        if (isNil(objects)) {
            return undefined;
        }
        return objects.map((/**
         * @param {?} object
         * @return {?}
         */
        object => new TreeviewItem({ text: object[textField], value: object })));
    }
}
TreeviewPipe.decorators = [
    { type: Pipe, args: [{
                name: 'rsTreeview'
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class RSTreeviewModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: RSTreeviewModule,
            providers: [
                TreeviewConfig,
                { provide: TreeviewI18n, useClass: TreeviewI18nDefault },
                { provide: TreeviewEventParser, useClass: DefaultTreeviewEventParser }
            ]
        };
    }
}
RSTreeviewModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    FormsModule,
                    CommonModule
                ],
                declarations: [
                    TreeviewComponent,
                    TreeviewItemComponent,
                    TreeviewPipe,
                    DropdownDirective,
                    DropdownMenuDirective,
                    DropdownToggleDirective,
                    DropdownTreeviewComponent
                ], exports: [
                    TreeviewComponent,
                    TreeviewItemComponent,
                    TreeviewPipe,
                    DropdownTreeviewComponent
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { RSTreeviewModule, TreeviewComponent, TreeviewConfig, TreeviewItem, TreeviewItemComponent as ɵa, TreeviewI18n as ɵb, TreeviewI18nDefault as ɵc, TreeviewEventParser as ɵd, DefaultTreeviewEventParser as ɵe, TreeviewPipe as ɵf, DropdownDirective as ɵg, DropdownMenuDirective as ɵh, DropdownToggleDirective as ɵi, DropdownTreeviewComponent as ɵj };
//# sourceMappingURL=rs-dropdown-tree.js.map
